import { Operand } from '3xpr';
import { Source, EntityMapping, Field, Sentence, From, Join, Page, Insert, Update, Delete, BulkInsert, MappingConfigService } from 'lambdaorm-base';
import { Query } from '../../../../query/domain';
import { OrmH3lp } from '../../../../shared/application';
import { DialectService } from '../../../../language/application';
import { DmlBuilderPort } from '../../../application';
export declare abstract class DmlBuilderAdapter implements DmlBuilderPort {
    protected readonly source: Source;
    protected readonly mapping: MappingConfigService;
    protected readonly dialect: DialectService;
    protected readonly helper: OrmH3lp;
    constructor(source: Source, mapping: MappingConfigService, dialect: DialectService, helper: OrmH3lp);
    build(sentence: Sentence): Query;
    protected buildSentence(sentence: Sentence): string;
    protected buildSelectSentence(sentence: Sentence): string;
    protected buildInsertSentence(sentence: Sentence): string;
    protected buildUpdateSentence(sentence: Sentence): string;
    protected buildDeleteSentence(sentence: Sentence): string;
    protected buildJoins(_entity: EntityMapping, joins: Join[]): string;
    protected buildFrom(from: From): string;
    protected buildInsert(operand: Insert | BulkInsert, entity: EntityMapping): string;
    protected buildUpdate(operand: Update, entity: EntityMapping): string;
    protected buildDelete(operand: Delete, entity: EntityMapping): string;
    protected buildPage(sentence: string, operand: Page): string;
    protected buildOperand(operand: Operand): string;
    protected buildArrowFunction(operand: Operand): string;
    protected buildFunctionRef(operand: Operand): string;
    protected buildOperator(operand: Operand): string;
    protected buildBlock(operand: Operand): string;
    protected buildObject(operand: Operand): string;
    protected buildList(operand: Operand): string;
    protected buildKeyValue(operand: Operand): string;
    protected buildField(field: Field): string;
    protected buildVariable(operand: Operand): string;
    protected buildConstant(operand: Operand): string;
}
